// Function to check if content script is already injected
async function isContentScriptInjected(tabId) {
  try {
    await chrome.scripting.executeScript({
      target: {tabId},
      func: () => true
    })
    return true
  } catch (e) {
    return false
  }
}

// Function to check initialization state
async function checkInitializationState(tabId) {
  try {
    const [{result}] = await chrome.scripting.executeScript({
      target: {tabId},
      func: () => {
        return new Promise(resolve => {
          chrome.runtime.sendMessage({type: 'GET_INIT_STATE'}, response => {
            resolve(response?.isInitializing || false)
          })
        })
      }
    })
    return result
  } catch (e) {
    return false
  }
}

// Debounce function
function debounce(func, wait) {
  let timeout
  return function executedFunction(...args) {
    const later = () => {
      clearTimeout(timeout)
      func(...args)
    }
    clearTimeout(timeout)
    timeout = setTimeout(later, wait)
  }
}

// Function to inject content script if needed
async function injectContentScriptIfNeeded(tabId) {
  if (!tabId) return

  console.log('Checking if content script needs to be injected...')
  const isInjected = await isContentScriptInjected(tabId)

  if (!isInjected) {
    console.log('Content script not injected, injecting...')
    try {
      await chrome.scripting.executeScript({
        target: {tabId},
        files: ['content.js']
      })
      console.log('Content script injected successfully')
    } catch (error) {
      console.error('Error injecting content script:', error)
    }
    return
  }

  // If script is already injected, send recalculate message
  console.log('Content script already injected, sending recalculate message...')
  try {
    await chrome.tabs.sendMessage(tabId, {action: 'recalculateMetrics'})
    console.log('Recalculate message sent successfully')
  } catch (error) {
    console.error('Error sending recalculate message:', error)
    // If we got an error, the content script might have been unloaded
    // Try to inject it again
    try {
      await chrome.scripting.executeScript({
        target: {tabId},
        files: ['content.js']
      })
      console.log('Content script re-injected successfully')
      await chrome.tabs.sendMessage(tabId, {action: 'recalculateMetrics'})
    } catch (error) {
      console.error('Error re-injecting content script:', error)
    }
  }
}

// Function to check if URL is analytics page
function isAnalyticsPage(url) {
  return url.match(/https:\/\/plugins\.jetbrains\.com\/plugin\/.*\/edit\/analytics/)
}

// Store initialization state for each tab
const tabInitializationState = new Map()

// Listen for messages from content script
chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
  console.log('Background script received message:', request)

  if (request.action === 'getCookie') {
    chrome.cookies.get({
      url: 'https://plugins.jetbrains.com',
      name: request.name
    }, cookie => {
      console.log('Got cookie:', cookie)
      sendResponse({
        success: true,
        value: cookie ? cookie.value : null
      })
    })
    return true // Will respond asynchronously
  }

  if (request.action === 'injectContentScript') {
    console.log('Injecting content script...')
    chrome.tabs.query({active: true, currentWindow: true}, function (tabs) {
      injectContentScriptIfNeeded(tabs[0].id)
    })
  }

  if (request.type === 'GET_INIT_STATE') {
    const tabId = sender.tab.id
    sendResponse({isInitializing: tabInitializationState.get(tabId) || false})
  } else if (request.type === 'SET_INIT_STATE') {
    const tabId = sender.tab.id
    tabInitializationState.set(tabId, request.isInitializing)
    sendResponse({success: true})
  }
  return true // Keep message channel open for async response
})

let authToken = null

// Intercept all API requests
chrome.webRequest.onBeforeSendHeaders.addListener(
  function (details) {
    if (details.url.includes('plugins.jetbrains.com/api/')) {
      // Look for Authorization header
      const authHeader = details.requestHeaders.find(h =>
        h.name.toLowerCase() === 'authorization'
      )

      if (authHeader && authHeader.value.startsWith('Bearer ')) {
        authToken = authHeader.value.substring(7) // Remove 'Bearer '
      }
    }
    return {requestHeaders: details.requestHeaders}
  },
  {urls: ['*://plugins.jetbrains.com/api/*']},
  ['requestHeaders']
)

// Respond to messages from content script
chrome.runtime.onMessage.addListener(
  function (request, sender, sendResponse) {
    if (request.type === 'GET_AUTH_TOKEN') {
      sendResponse({token: authToken})
    }
  }
)

// Activate rules on extension installation
chrome.runtime.onInstalled.addListener(() => {
  console.log('Extension installed, activating rules')
})

// Track last processed URL and tab
let lastProcessedUrls = new Map()

// Debounced inject function
const debouncedInject = debounce(async (tabId, url) => {
  console.log('Processing URL:', url, 'for tab:', tabId)
  await injectContentScriptIfNeeded(tabId)
  // Update last processed URL after successful injection
  lastProcessedUrls.set(tabId, url)
}, 500) // 500ms debounce

// Handle regular navigation
chrome.tabs.onUpdated.addListener((tabId, changeInfo, tab) => {
  // Only process if the page has finished loading and has a URL
  if (changeInfo.status === 'complete' && tab.url) {
    // Check if this is an analytics page
    if (isAnalyticsPage(tab.url)) {
      console.log('Regular navigation to analytics page detected')
      debouncedInject(tabId, tab.url)
    }
  }
})

// Handle SPA navigation
chrome.webNavigation.onHistoryStateUpdated.addListener((details) => {
  // Check if this is an analytics page
  if (!isAnalyticsPage(details.url)) {
    return
  }

  console.log('SPA navigation to analytics page detected')
  debouncedInject(details.tabId, details.url)
})

// Clean up when tab is closed
chrome.tabs.onRemoved.addListener((tabId) => {
  lastProcessedUrls.delete(tabId)
  tabInitializationState.delete(tabId)
})
