// Cleanup any existing instance
if (window.revenueMetricsCleanup) {
  window.revenueMetricsCleanup()
}

// Debounce function
function debounce(func, wait) {
  let timeout
  return function executedFunction(...args) {
    const later = () => {
      clearTimeout(timeout)
      func(...args)
    }
    clearTimeout(timeout)
    timeout = setTimeout(later, wait)
  }
}

// Function to check if current page is analytics page
function isAnalyticsPage() {
  return window.location.href.includes('/edit/analytics')
}

// Initialize logger
const logger = {
  debug: (...args) => console.debug('[Revenue Metrics]', ...args),
  info: (...args) => console.log('[Revenue Metrics]', ...args),
  warn: (...args) => console.warn('[Revenue Metrics]', ...args),
  error: (...args) => console.error('[Revenue Metrics]', ...args)
}

// Initialize extension
const initializeExtensionDebounced = debounce(async function () {
  // Skip if not on analytics page
  if (!isAnalyticsPage()) {
    console.log('Not on analytics page, skipping initialization')
    return
  }

  console.log('Initializing extension...')
  console.log('Current URL:', window.location.href)
  console.log('Document ready state:', document.readyState)

  // Check if already initializing
  const isInitializing = await getInitializationState()
  if (isInitializing) {
    console.log('Already initializing, skipping...')
    return
  }

  // Set initializing state
  await setInitializationState(true)

  try {
    // Wait for DOM to be ready
    if (document.readyState === 'loading') {
      await new Promise(resolve => {
        document.addEventListener('DOMContentLoaded', resolve, {once: true})
      })
    }

    await calculateMRR()
  } catch (error) {
    console.error('Error during initialization:', error)
  } finally {
    // Reset initializing state
    await setInitializationState(false)
  }

  // Create cleanup function
  window.revenueMetricsCleanup = () => {
    if (currentController) {
      currentController.abort()
      currentController = null
    }
    if (currentMetricsInterval) {
      clearInterval(currentMetricsInterval)
      currentMetricsInterval = null
    }
    cleanupHandlers.cleanup()
    setInitializationState(false)
  }
}, 500) // 500ms debounce

// Function to get the token with retries
async function getAuthToken(maxAttempts = 10) {
  for (let attempt = 0; attempt < maxAttempts; attempt++) {
    const token = await new Promise((resolve) => {
      chrome.runtime.sendMessage({type: 'GET_AUTH_TOKEN'}, response => {
        resolve(response?.token)
      })
    })

    if (token) {
      return token
    }

    // Wait before next attempt
    if (attempt < maxAttempts - 1) {
      await new Promise(resolve => setTimeout(resolve, 500))
    }
  }

  throw new Error('Could not get auth token after ' + maxAttempts + ' attempts')
}

// Store current AbortController
let currentController = null

// Function to make authenticated API request
async function makeApiRequest(endpoint) {
  try {
    // Cancel any existing request
    if (currentController) {
      currentController.abort()
    }

    // Create new controller
    currentController = new AbortController()

    const token = await getAuthToken()
    const response = await fetch(`https://plugins.jetbrains.com/api/${endpoint}`, {
      headers: {
        'Authorization': `Bearer ${token}`,
        'Content-Type': 'application/json'
      },
      signal: currentController.signal
    })

    if (!response.ok) {
      throw new Error(`API request failed: ${response.status}`)
    }

    return await response.json()
  } catch (error) {
    if (error.name === 'AbortError') {
      logger.log('Request was cancelled')
      return null
    }
    throw error
  }
}

// Function to wait for element with timeout
async function waitForElement(selector, timeout = 5000) {
  return new Promise((resolve, reject) => {
    const element = document.querySelector(selector)
    if (element) {
      resolve(element)
      return
    }

    const observer = new MutationObserver((mutations, obs) => {
      const element = document.querySelector(selector)
      if (element) {
        obs.disconnect()
        resolve(element)
      }
    })

    cleanupHandlers.addObserver(observer)
    observer.observe(document.body, {
      childList: true,
      subtree: true
    })

    const timer = setTimeout(() => {
      observer.disconnect()
      reject(new Error(`Timeout waiting for ${selector}`))
    }, timeout)

    cleanupHandlers.addTimer(timer)
  })
}

// Check initialization state from background
async function getInitializationState() {
  return new Promise((resolve) => {
    chrome.runtime.sendMessage({type: 'GET_INIT_STATE'}, (response) => {
      resolve(response.isInitializing)
    })
  })
}

// Set initialization state in background
async function setInitializationState(isInitializing) {
  return new Promise((resolve) => {
    chrome.runtime.sendMessage({type: 'SET_INIT_STATE', isInitializing}, (response) => {
      resolve(response.success)
    })
  })
}

async function calculateMRR() {
  console.log('=== Starting MRR calculation ===')

  // Show placeholder
  displayMetrics('loading', 'loading', 'loading', 'loading', 'loading', 'loading', 'loading')

  // Get current date for all calculations
  const now = new Date()
  const thirtyDaysAgo = new Date(now)
  thirtyDaysAgo.setDate(thirtyDaysAgo.getDate() - 30)
  // Note: we keep the current time for more precise 30-day period calculation

  // Get plugin ID from URL
  const match = window.location.pathname.match(/\/plugin\/(\d+)/)
  const pluginId = match ? match[1] : null
  console.log('Plugin ID:', pluginId)

  if (!pluginId) {
    console.error('Could not determine plugin ID from URL')
    return
  }

  // Get all sales for the last 24 months to account for annual subscriptions
  console.log('Fetching sales data...')

  const allSales = []

  // Create array of dates for requests (24 months)
  const dates = []
  const date = new Date()
  for (let i = 0; i < 24; i++) {
    date.setMonth(date.getMonth() - i)
    const year = date.getFullYear()
    const month = date.getMonth() + 1
    const lastDay = new Date(year, month, 0).getDate()

    dates.push({
      beginDate: `${year}-${month.toString().padStart(2, '0')}-01`,
      endDate: `${year}-${month.toString().padStart(2, '0')}-${lastDay}`
    })
  }

  // Make requests for each month
  try {
    for (const period of dates) {
      // Create URL for request
      const url = `marketplace/plugin/${pluginId}/sales-info?beginDate=${period.beginDate}&endDate=${period.endDate}`

      const response = await makeApiRequest(url)
      if (!response) {
        console.log('API request failed')
        continue
      }

      allSales.push(...response)
    }
  } catch (error) {
    console.error('Error fetching sales data:', error)
    await displayMetrics('error', 'error', 'error', 'error', 'error', 'error', 'error')
    return
  }

  console.log('Total sales loaded:', allSales.length)

  // Calculate MRR
  if (!Array.isArray(allSales)) {
    console.log('No sales data available')
    return
  }

  console.log('\n=== Processing sales ===')
  console.log('Total sales to process:', allSales.length)

  // Initialize all metrics
  let currentMonthMRR = 0
  let previousMonthMRR = 0
  let churnedMRR = 0
  let expansionMRR = 0
  let processedSubscriptions = 0
  let activePayingCustomers = new Set()

  // Statistics for analysis
  const stats = {
    salesByType: {
      monthly: [],
      yearly: [],
      perpetual: [],
      unknown: []
    },
    amountStats: {
      monthly: {min: Infinity, max: -Infinity, total: 0, count: 0},
      yearly: {min: Infinity, max: -Infinity, total: 0, count: 0},
      perpetual: {min: Infinity, max: -Infinity, total: 0, count: 0}
    },
    currencyStats: {},
    discountStats: {
      full: 0,  // 100% discount
      partial: 0,  // partial discount
      none: 0  // no discount
    },
    zeroAmountReasons: {
      noAmount: 0,
      noPeriod: 0,
      noLineItems: 0,
      emptyLineItems: 0,
      zeroLineItemAmount: 0
    },
    subscriptionStatus: {
      active: 0,
      inactive: 0,
      lastRecord: 0,
      notLastRecord: 0
    }
  }

  // Group records by subscription ID
  const subscriptionGroups = new Map() // subscriptionId -> [sales]
  allSales.forEach(sale => {
    if (!sale.lineItems?.[0]?.licenseIds?.[0]) {
      console.log('Missing license ID:', sale)
      return
    }
    // Use the first licenseId as the subscription ID
    const subscriptionId = sale.lineItems[0].licenseIds[0]
    if (!subscriptionGroups.has(subscriptionId)) {
      subscriptionGroups.set(subscriptionId, [])
    }
    subscriptionGroups.get(subscriptionId).push(sale)
  })

  // Sort records in each group by start date
  subscriptionGroups.forEach((sales) => {
    sales.sort((a, b) => {
      const aDate = new Date(a.lineItems[0].subscriptionDates.start[0],
        a.lineItems[0].subscriptionDates.start[1] - 1,
        a.lineItems[0].subscriptionDates.start[2])
      const bDate = new Date(b.lineItems[0].subscriptionDates.start[0],
        b.lineItems[0].subscriptionDates.start[1] - 1,
        b.lineItems[0].subscriptionDates.start[2])
      return aDate - bDate
    })
  })

  console.log('\n=== Processing subscription groups ===')
  console.log('Total subscription groups:', subscriptionGroups.size)

  // Process each record
  allSales.forEach(sale => {
    if (!sale.amountUSD || !sale.period) {
      return
    }

    const monthlyAmount = calculateMonthlyAmount(sale)

    // Check for subscription dates
    if (!sale.lineItems?.[0]?.subscriptionDates?.start || !sale.lineItems?.[0]?.subscriptionDates?.end) {
      console.log('Missing subscription dates:', {
        ref: sale.ref,
        period: sale.period,
        lineItems: sale.lineItems
      })
      return
    }

    // Get dates from the correct place in API and convert to Moscow timezone
    const subscriptionDates = sale.lineItems[0].subscriptionDates
    const moscowFormatter = new Intl.DateTimeFormat('en-US', {
      timeZone: 'Europe/Moscow',
      year: 'numeric',
      month: '2-digit',
      day: '2-digit',
      hour: '2-digit',
      minute: '2-digit',
      second: '2-digit',
      hour12: false
    })

    // Create dates in Moscow timezone
    const startDate = new Date(
      moscowFormatter.format(new Date(
        subscriptionDates.start[0],
        subscriptionDates.start[1] - 1,
        subscriptionDates.start[2],
        0, 0, 0
      ))
    )

    const endDate = new Date(
      moscowFormatter.format(new Date(
        subscriptionDates.end[0],
        subscriptionDates.end[1] - 1,
        subscriptionDates.end[2],
        23, 59, 59
      ))
    )

    // Check customer type
    const isOrganization = sale.customer?.type === 'Organization'

    // Check if this is the latest record for this subscription
    const licenseId = sale.lineItems[0].licenseIds[0]
    const subscriptionRecords = subscriptionGroups.get(licenseId)
    const currentRecordIndex = subscriptionRecords.indexOf(sale)
    const isLastRecord = currentRecordIndex === subscriptionRecords.length - 1

    // A subscription is considered active only if it hasn't ended yet
    const isActiveNow = endDate >= now

    // For personal subscriptions, we only count the last record
    // For organizational subscriptions, we count all active records
    const shouldCountNow = isOrganization ? isActiveNow : (isLastRecord && isActiveNow)

    // Calendar month checks - for previous month
    const lastMonthDate = new Date(now.getFullYear(), now.getMonth() - 1, now.getDate())
    const wasActiveLastMonth = startDate <= lastMonthDate && endDate >= lastMonthDate

    // If the subscription was active last month, add to previousMonthMRR
    if (wasActiveLastMonth) {
      previousMonthMRR += monthlyAmount
      // Subscription is considered churned if it was active last month but not active now
      if (!shouldCountNow) {
        churnedMRR += monthlyAmount
      }
    }

    // If active now, add to current month metrics
    if (shouldCountNow) {
      currentMonthMRR += monthlyAmount
      // If the subscription was not active last month, it's new revenue
      if (!wasActiveLastMonth) {
        expansionMRR += monthlyAmount
      }
      if (sale.customer) {
        activePayingCustomers.add(sale.customer.code) // Add customer with active subscription
      }
    }
    processedSubscriptions++

    // Update statistics
    const period = sale.period.toLowerCase()
    if (period === 'monthly') {
      stats.salesByType.monthly.push(sale)
      stats.amountStats.monthly.min = Math.min(stats.amountStats.monthly.min, monthlyAmount)
      stats.amountStats.monthly.max = Math.max(stats.amountStats.monthly.max, monthlyAmount)
      stats.amountStats.monthly.total += monthlyAmount
      stats.amountStats.monthly.count++
    } else if (period === 'annual') {
      stats.salesByType.yearly.push(sale)
      stats.amountStats.yearly.min = Math.min(stats.amountStats.yearly.min, monthlyAmount)
      stats.amountStats.yearly.max = Math.max(stats.amountStats.yearly.max, monthlyAmount)
      stats.amountStats.yearly.total += monthlyAmount
      stats.amountStats.yearly.count++
    } else if (period === 'perpetual') {
      stats.salesByType.perpetual.push(sale)
      stats.amountStats.perpetual.min = Math.min(stats.amountStats.perpetual.min, monthlyAmount)
      stats.amountStats.perpetual.max = Math.max(stats.amountStats.perpetual.max, monthlyAmount)
      stats.amountStats.perpetual.total += monthlyAmount
      stats.amountStats.perpetual.count++
    } else {
      stats.salesByType.unknown.push(sale)
    }

    // Update currency statistics
    const currency = sale.currency
    if (!stats.currencyStats[currency]) {
      stats.currencyStats[currency] = 0
    }
    stats.currencyStats[currency] += monthlyAmount

    // Update discount statistics
    if (sale.discount === 100) {
      stats.discountStats.full++
    } else if (sale.discount > 0) {
      stats.discountStats.partial++
    } else {
      stats.discountStats.none++
    }

    // Update zero amount reasons
    if (!sale.amountUSD) {
      stats.zeroAmountReasons.noAmount++
    } else if (!sale.period) {
      stats.zeroAmountReasons.noPeriod++
    } else if (!sale.lineItems || sale.lineItems.length === 0) {
      stats.zeroAmountReasons.noLineItems++
    } else if (sale.lineItems[0].amountUSD === 0) {
      stats.zeroAmountReasons.zeroLineItemAmount++
    }

    // Update subscription status
    if (shouldCountNow) {
      stats.subscriptionStatus.active++
    } else {
      stats.subscriptionStatus.inactive++
    }
    if (isLastRecord) {
      stats.subscriptionStatus.lastRecord++
    } else {
      stats.subscriptionStatus.notLastRecord++
    }
  })

  console.log('\n=== Detailed MRR Breakdown ===')
  console.log('Current Month MRR:', currentMonthMRR.toFixed(2))
  console.log('Previous Month MRR:', previousMonthMRR.toFixed(2))
  console.log('Churned MRR:', churnedMRR.toFixed(2))
  console.log('Expansion MRR:', expansionMRR.toFixed(2))
  console.log('Active Paying Customers:', activePayingCustomers.size)
  console.log('Processed Subscriptions:', processedSubscriptions)

  console.log('\n=== Statistics ===')
  console.log('Sales by Type:')
  console.log('  Monthly:', stats.salesByType.monthly.length)
  console.log('  Yearly:', stats.salesByType.yearly.length)
  console.log('  Perpetual:', stats.salesByType.perpetual.length)
  console.log('  Unknown:', stats.salesByType.unknown.length)
  console.log('Amount Statistics:')
  console.log('  Monthly:', {
    min: stats.amountStats.monthly.min.toFixed(2),
    max: stats.amountStats.monthly.max.toFixed(2),
    total: stats.amountStats.monthly.total.toFixed(2),
    count: stats.amountStats.monthly.count
  })
  console.log('  Yearly:', {
    min: stats.amountStats.yearly.min.toFixed(2),
    max: stats.amountStats.yearly.max.toFixed(2),
    total: stats.amountStats.yearly.total.toFixed(2),
    count: stats.amountStats.yearly.count
  })
  console.log('  Perpetual:', {
    min: stats.amountStats.perpetual.min.toFixed(2),
    max: stats.amountStats.perpetual.max.toFixed(2),
    total: stats.amountStats.perpetual.total.toFixed(2),
    count: stats.amountStats.perpetual.count
  })
  console.log('Currency Statistics:')
  for (const currency in stats.currencyStats) {
    console.log(`  ${currency}: ${stats.currencyStats[currency].toFixed(2)}`)
  }
  console.log('Discount Statistics:')
  console.log('  Full Discount:', stats.discountStats.full)
  console.log('  Partial Discount:', stats.discountStats.partial)
  console.log('  No Discount:', stats.discountStats.none)
  console.log('Zero Amount Reasons:')
  console.log('  No Amount:', stats.zeroAmountReasons.noAmount)
  console.log('  No Period:', stats.zeroAmountReasons.noPeriod)
  console.log('  No Line Items:', stats.zeroAmountReasons.noLineItems)
  console.log('  Zero Line Item Amount:', stats.zeroAmountReasons.zeroLineItemAmount)
  console.log('Subscription Status:')
  console.log('  Active:', stats.subscriptionStatus.active)
  console.log('  Inactive:', stats.subscriptionStatus.inactive)
  console.log('  Last Record:', stats.subscriptionStatus.lastRecord)
  console.log('  Not Last Record:', stats.subscriptionStatus.notLastRecord)

  console.log('\n=== Key Performance Indicators ===')
  console.log('Active Rate:', ((stats.subscriptionStatus.active / processedSubscriptions) * 100).toFixed(1) + '%')
  console.log('Renewal Rate:', ((1 - (churnedMRR / previousMonthMRR)) * 100).toFixed(1) + '%')
  console.log('Growth Rate:', (((currentMonthMRR - previousMonthMRR) / previousMonthMRR) * 100).toFixed(1) + '%')
  console.log('Monthly vs Annual Split:', {
    monthly: ((stats.salesByType.monthly.length / processedSubscriptions) * 100).toFixed(1) + '%',
    annual: ((stats.salesByType.yearly.length / processedSubscriptions) * 100).toFixed(1) + '%'
  })

  console.log('\n=== Final Metrics ===')
  console.log('Current Month MRR:', currentMonthMRR.toFixed(2))
  console.log('Previous Month MRR:', previousMonthMRR.toFixed(2))
  console.log('Churn Rate:', ((churnedMRR / previousMonthMRR) * 100).toFixed(1) + '%')

  // Pass ARPU for paying customers and growth metrics to display function
  displayMetrics(
    currentMonthMRR.toFixed(2),
    (currentMonthMRR / activePayingCustomers.size).toFixed(2),
    activePayingCustomers.size,
    ((churnedMRR / previousMonthMRR) * 100).toFixed(1),
    expansionMRR.toFixed(2),
    churnedMRR.toFixed(2),
    previousMonthMRR.toFixed(2)
  )
}

// Store current metrics update interval
let currentMetricsInterval = null

// Function to display metrics on the page
async function displayMetrics(mrr, arpu, activePayingCustomers, calendarChurnRate, expansionMRR, churnedMRR, previousMonthMRR) {
  console.log('Displaying metrics:', {
    mrr,
    arpu,
    activePayingCustomers,
    calendarChurnRate,
    expansionMRR,
    churnedMRR,
    previousMonthMRR
  })

  const maxAttempts = 10
  let attempt = 0

  // Clear previous interval if exists
  if (currentMetricsInterval) {
    clearInterval(currentMetricsInterval)
  }
  // Store numeric values for calculations
  const mrrNum = parseFloat(mrr)
  const arpuNum = parseFloat(arpu)
  const calendarChurnRateNum = parseFloat(calendarChurnRate)
  const expansionMRRNum = parseFloat(expansionMRR)
  const churnedMRRNum = parseFloat(churnedMRR)
  const previousMonthMRRNum = parseFloat(previousMonthMRR)

  // Calculate derived values
  const netMRRGrowth = expansionMRRNum - churnedMRRNum
  const netMRRGrowthFormatted = netMRRGrowth.toFixed(2)
  const netMRRGrowthPercentage = ((netMRRGrowth / previousMonthMRRNum) * 100).toFixed(2)
  const expansionMRRPercentage = ((expansionMRRNum / previousMonthMRRNum) * 100).toFixed(2)

  currentMetricsInterval = setInterval(() => {
    attempt++
    console.log('Looking for analytics-header_right, attempt:', attempt)

    const headerRight = document.querySelector('.analytics-header_right')
    if (!headerRight) {
      if (attempt > maxAttempts) {
        clearInterval(currentMetricsInterval)
        console.log('Could not find analytics-header_right after', maxAttempts, 'attempts')
      }
      return
    }

    // Check if metrics container exists
    let metricsContainer = headerRight.querySelector('#cascade-metrics')
    if (!metricsContainer) {
      metricsContainer = document.createElement('div')
      metricsContainer.id = 'cascade-metrics'
      metricsContainer.style = 'margin-top: 24px; margin-bottom: 24px;'
      headerRight.appendChild(metricsContainer)
    }

    // Show loading placeholder if data is still loading
    if (mrr === 'loading') {
      metricsContainer.innerHTML = `
        <div class="wt-row wt-row_size_xs wt-row_wide wt-row_wrap wt-row_justify_between wt-row_align-items_end rs-text-3 rs-text-3_theme_light" style="align-items: self-start;">
          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">Current MRR</div>
            <h3 class="wt-h3" style="color: var(--gray-color, #6c757d);">Loading...</h3>
          </div>
          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">ARPU</div>
            <h3 class="wt-h3" style="color: var(--gray-color, #6c757d);">Loading...</h3>
          </div>
          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">Active Subscriptions</div>
            <h3 class="wt-h3" style="color: var(--gray-color, #6c757d);">Loading...</h3>
          </div>
        </div>

        <div class="wt-row wt-row_size_xs wt-row_wide wt-row_wrap wt-row_justify_between wt-row_align-items_end rs-text-3 rs-text-3_theme_light" style="align-items: self-start;">
          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">Expansion MRR</div>
            <h3 class="wt-h3" style="color: var(--gray-color, #6c757d);">Loading...</h3>
          </div>
          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">Churned MRR</div>
            <h3 class="wt-h3" style="color: var(--gray-color, #6c757d);">Loading...</h3>
          </div>
          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">Net MRR Growth</div>
            <h3 class="wt-h3" style="color: var(--gray-color, #6c757d);">Loading...</h3>
          </div>
        </div>
      `
      clearInterval(currentMetricsInterval)
      return
    }

    // Verify all values are calculated
    if (!mrrNum || !arpuNum || !expansionMRRNum || !churnedMRRNum || !calendarChurnRateNum || !previousMonthMRRNum) {
      if (attempt > maxAttempts) {
        clearInterval(currentMetricsInterval)
        console.log('Metrics not ready after', maxAttempts, 'attempts')
      }
      return
    }

    console.log('All metrics are ready, updating display')

    // Update container with real metrics
    metricsContainer.innerHTML = `
      <div class="wt-row wt-row_size_xs wt-row_wide wt-row_wrap wt-row_justify_between wt-row_align-items_end rs-text-3 rs-text-3_theme_light" style="align-items: self-start;">
          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">
              Current MRR
              <span title="Monthly Recurring Revenue (MRR) is a forecast of your regular monthly income based on currently active subscriptions.

Think of it as your business's 'temperature' right now - how much money you would make if everything stays the same.

Current MRR: $${mrr}
MRR per subscriber: $${arpu}

Keep in mind that actual next month's revenue may differ due to:
• Subscription cancellations
• Plan upgrades
• New customers
• Annual subscriptions ending" style="cursor: pointer; margin-left: 4px; opacity: 0.7;">
                <svg viewBox="0 0 24 24" style="width: 14px; height: 14px;"><path fill="currentColor" d="M12 21a9 9 0 100-17.999A9 9 0 0012 21zM10.546 7.563a1.5 1.5 0 112.893.79 1.5 1.5 0 01-2.893-.79zM10 12.494V11.5h3v6.495h-2V12.5h-1v-.006z"></path></svg>
              </span>
            </div>
            <h3 class="wt-h3">$${mrr}</h3>
          </div>
          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">
              ARPU
              <span title="Average Revenue Per User (ARPU) shows how much revenue you generate per customer monthly.

Formula: Current MRR / Number of Active Paying Customers
Current calculation: $${mrr} / ${activePayingCustomers} = $${arpu}

An increasing ARPU typically indicates successful upselling or a shift toward higher-value customers." style="cursor: pointer; margin-left: 4px; opacity: 0.7;">
                <svg viewBox="0 0 24 24" style="width: 14px; height: 14px;"><path fill="currentColor" d="M12 21a9 9 0 100-17.999A9 9 0 0012 21zM10.546 7.563a1.5 1.5 0 112.893.79 1.5 1.5 0 01-2.893-.79zM10 12.494V11.5h3v6.495h-2V12.5h-1v-.006z"></path></svg>
              </span>
            </div>
            <h3 class="wt-h3">$${arpu}</h3>
          </div>

          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">
              Active customers
              <span title="The total count of paying customers with active customers.

Current active customers: ${activePayingCustomers}

This core metric directly affects your MRR and serves as a key indicator of market penetration." style="cursor: pointer; margin-left: 4px; opacity: 0.7;">
                <svg viewBox="0 0 24 24" style="width: 14px; height: 14px;"><path fill="currentColor" d="M12 21a9 9 0 100-17.999A9 9 0 0012 21zM10.546 7.563a1.5 1.5 0 112.893.79 1.5 1.5 0 01-2.893-.79zM10 12.494V11.5h3v6.495h-2V12.5h-1v-.006z"></path></svg>
              </span>
            </div>
            <h3 class="wt-h3">${activePayingCustomers}</h3>
          </div>
        </div>

        <div class="wt-row wt-row_size_xs wt-row_wide wt-row_wrap wt-row_justify_between wt-row_align-items_end rs-text-3 rs-text-3_theme_light" style="align-items: self-start;">
          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">
              Expansion MRR
              <span title="Expansion MRR represents additional recurring revenue from both new customers and existing customers who upgraded their subscriptions.

Current expansion: +$${expansionMRR}
As percentage of Previous MRR: ${expansionMRRPercentage}%

This metric is crucial for understanding your business growth from both customer acquisition and customer expansion." style="cursor: pointer; margin-left: 4px; opacity: 0.7;">
                <svg viewBox="0 0 24 24" style="width: 14px; height: 14px;"><path fill="currentColor" d="M12 21a9 9 0 100-17.999A9 9 0 0012 21zM10.546 7.563a1.5 1.5 0 112.893.79 1.5 1.5 0 01-2.893-.79zM10 12.494V11.5h3v6.495h-2V12.5h-1v-.006z"></path></svg>
              </span>
            </div>
            <h3 class="wt-h3" style="color: var(--green-color, #28a745);">+$${expansionMRR} <span style="font-size: 14px; opacity: 0.8;">(${expansionMRRPercentage}%)</span></h3>
          </div>
          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">
              Churned MRR
              <span title="Churned MRR shows how much monthly recurring revenue was lost from customers who cancelled or downgraded their subscriptions.

Formula: Sum of MRR from subscriptions that were active last month but not active this month.
Current calculation: $${churnedMRR} (${calendarChurnRate}% of Previous Month MRR)" style="cursor: pointer; margin-left: 4px; opacity: 0.7;">
<svg viewBox="0 0 24 24" style="width: 14px; height: 14px;"><path fill="currentColor" d="M12 21a9 9 0 100-17.999A9 9 0 0012 21zM10.546 7.563a1.5 1.5 0 112.893.79 1.5 1.5 0 01-2.893-.79zM10 12.494V11.5h3v6.495h-2V12.5h-1v-.006z"></path></svg>
              </span>
            </div>
            <h3 class="wt-h3" style="color: var(--red-color, #dc3545);">$${churnedMRR} <span style="font-size: 14px; opacity: 0.8;">(${calendarChurnRate}%)</span></h3>
          </div>
          <div class="wt-col-3 wt-offset-top-md-12 wt-col-lg-5 wt-col-sm-12">
            <div style="display: flex; align-items: center;">
              Net MRR Growth
              <span title="Net MRR Growth (or Net New MRR) shows the difference between gained and lost revenue.

Formula: Expansion MRR - Churned MRR
Current calculation: $${expansionMRR} - $${churnedMRR} = ${netMRRGrowth >= 0 ? '+' : ''}\$${netMRRGrowthFormatted}
As percentage of Previous MRR: ${netMRRGrowthPercentage}%

A consistently positive number indicates healthy business growth." style="cursor: pointer; margin-left: 4px; opacity: 0.7;">
                <svg viewBox="0 0 24 24" style="width: 14px; height: 14px;"><path fill="currentColor" d="M12 21a9 9 0 100-17.999A9 9 0 0012 21zM10.546 7.563a1.5 1.5 0 112.893.79 1.5 1.5 0 01-2.893-.79zM10 12.494V11.5h3v6.495h-2V12.5h-1v-.006z"></path></svg>
              </span>
            </div>
            <h3 class="wt-h3" style="color: ${netMRRGrowth >= 0 ? 'var(--green-color, #28a745)' : 'var(--red-color, #dc3545)'};">
              ${netMRRGrowth >= 0 ? '+' : ''}\$${netMRRGrowthFormatted} <span style="font-size: 14px; opacity: 0.8;">(${netMRRGrowthPercentage}%)</span>
            </h3>
          </div>
        </div>
      `
    clearInterval(currentMetricsInterval)
  }, 1000)

  // Store interval for cleanup
  cleanupHandlers.intervals.add(currentMetricsInterval)
}

// Helper function to calculate monthly amount
function calculateMonthlyAmount(sale) {
  if (!sale.amountUSD || !sale.period) return 0

  const amount = parseFloat(sale.amountUSD)
  if (isNaN(amount) || amount === 0) return 0

  const period = sale.period.toLowerCase()

  if (period === 'monthly') {
    return amount
  } else if (period === 'annual') {
    return amount / 12
  }

  return 0
}

// Store all timers and intervals
const cleanupHandlers = {
  timers: new Set(),
  intervals: new Set(),
  observers: new Set(),

  addTimer(timer) {
    this.timers.add(timer)
  },

  addInterval(interval) {
    this.intervals.add(interval)
  },

  addObserver(observer) {
    this.observers.add(observer)
  },

  cleanup() {
    // Clear all timers
    this.timers.forEach(timer => clearTimeout(timer))
    this.timers.clear()

    // Clear all intervals
    this.intervals.forEach(interval => clearInterval(interval))
    this.intervals.clear()

    // Disconnect all observers
    this.observers.forEach(observer => observer.disconnect())
    this.observers.clear()
  }
}

// Listen for messages from background script
chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
  if (request.type === 'GET_INIT_STATE') {
    getInitializationState().then(state => {
      sendResponse({isInitializing: state})
    })
    return true // Keep the message channel open for async response
  }

  if (request.action === 'recalculateMetrics') {
    console.log('Received request to recalculate metrics')
    initializeExtensionDebounced()
  }
})

// Initial initialization
initializeExtensionDebounced()
